import typer


class logger:
    def __init__():
        pass

    def info(message: str):
        typer.echo(
            f"{typer.style('[INFO]', fg=typer.colors.GREEN)}"
            + f"{typer.style(message, fg=typer.colors.WHITE)}"
        )

    def warning(message: str):
        typer.echo(
            f"{typer.style('[WARNING]', fg=typer.colors.YELLOW)}"
            + f"{typer.style(message, fg=typer.colors.WHITE)}"
        )

    def error(message: str):
        typer.echo(
            f"{typer.style('[ERROR]', fg=typer.colors.RED)}"
            + f"{typer.style(message, fg=typer.colors.WHITE)}",
            err=True,
        )

    def debug(message: str):
        typer.echo(
            f"{typer.style('[SUCCESS]', fg=typer.colors.MAGENTA)}"
            + f"{typer.style(message, fg=typer.colors.WHITE)}"
        )

    def critical(message: str):
        typer.echo(
            f"{typer.style('[CRITICAL]', fg=typer.colors.BRIGHT_RED)} "
            f"{typer.style(message, fg=typer.colors.WHITE, bold=True)}",
            err=True,
        )
