class BaseCdus:
    def provision_list(self, cdu_id, region):
        print(f"Calling provisioning list for ({cdu_id}) from Base Cdus class.")

    def provision_add(self, view_name, region):
        print(f"Calling provisioning add for ({view_name}) from Base Cdus class.")

    def provision_delete(self, view_name, region):
        print(f"Calling provisioning delete for ({view_name}) from Base Cdus class.")

    def provision_add_table(self, table_name, region):
        print(f"Calling provisioning add for ({table_name}) from Base Cdus class.")
